@@ -39,6 +39,7 @@
 #include "screen-review.h"
 #include <libxml/parser.h>
 
+
 gboolean login_time;
 
 static SRLClientHandle src_srl_client;
@@ -213,8 +214,9 @@ src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
 
     src_enable_format_string = TRUE; 
     sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-    if (strcmp (reason, "object:text-changed:insert") == 0
-	    && (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL))
+
+    if (strcmp (reason, "object:text-changed:insert") == 0 && role != SR_ROLE_TERMINAL
+	    && (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML))
     {
 	gchar *diff;
 	if (sro_text_get_difference (obj, &diff, SR_INDEX_CONTAINER))
@@ -255,20 +257,24 @@ src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
 	if (txt && txt[0])
 	{
 	    SRObjectRoles role;
-	    SRState state;
 	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	    sro_get_state (obj, &state, SR_INDEX_CONTAINER);
-	    if (state & SR_STATE_FOCUSED || strcmp (reason, "present") == 0)
+	    if (srl_is_object_focused (obj) || strcmp (reason, "present") == 0 ||
+		strcmp (reason, "object:context-switched") == 0)
 	    {
 		if ( role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
-		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
+		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
 		else if ((role == SR_ROLE_TABLE || role == SR_ROLE_TREE_TABLE) &&
 			strcmp (reason , "focus:") == 0)
 		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, FALSE);
 		else
-		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
+		{
+		    gboolean interrupt = TRUE;
+		    if (strcmp (reason, "object:text-changed:insert") == 0 && role == SR_ROLE_TERMINAL)
+			interrupt = FALSE;
+		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, interrupt);
+		}
 	    }
-	    else
+	    else if (srl_is_object_watched (obj))
 	    {
 		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
 	    }
@@ -379,6 +385,8 @@ src_present_crt_window ()
     	{
 	    if (strcmp (reason, "window:create") == 0)
 		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_WARNING, FALSE);
+	    else if (strcmp (reason, "window:rename") == 0)
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_LOW, TRUE);
 	    else
 		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
 	}
