@@ -620,14 +620,18 @@ srle_set_acc (SRLEvent *event)
 		Accessible_unref (parent);
 	}
     }
-    else if (srle_has_type (event, SRL_EVENT_FOCUS) &&
-	srle_acc_has_state (event, SPI_STATE_MANAGES_DESCENDANTS))
+    else if ((srle_has_type (event, SRL_EVENT_FOCUS) &&
+		    srle_acc_has_state (event, SPI_STATE_MANAGES_DESCENDANTS)) ||
+	     ( (srle_has_type (event, SRL_EVENT_SELECTION_CHANGED)  ||
+	           srle_has_type (event, SRL_EVENT_FOCUS)) &&
+		srle_acc_has_role (event, SPI_ROLE_LAYERED_PANE)))
     {
 	AccessibleSelection *selection;
 	selection = Accessible_getSelection ((Accessible*)srle_get_acc (event));
 	if (selection)
 	{
 	    gint i, cnt;
+	    gboolean is_focused_child = FALSE;
 	    cnt = AccessibleSelection_getNSelectedChildren (selection);
 	    for (i = 0; i < cnt; i++)
 	    {
@@ -638,12 +642,35 @@ srle_set_acc (SRLEvent *event)
 		    if (srl_acc_has_state (child, SPI_STATE_FOCUSED))
 		    {
 			event->acc = child;
+			is_focused_child = TRUE;
 			break;
 		    }
 		    else
 			Accessible_unref (child);
 		}
 	    }
+	    
+	    if (!is_focused_child && srle_acc_has_role (event, SPI_ROLE_TABLE))
+	    {
+		for (i = 0; i < cnt; i++)
+		{
+		    Accessible *child;
+		    child = AccessibleSelection_getSelectedChild (selection, i);
+		    if (child)
+		    {
+			if (Accessible_getRole (child) == SPI_ROLE_TABLE_CELL &&
+			    srl_acc_has_state (child, SPI_STATE_SHOWING) &&
+			    srl_acc_has_state (child, SPI_STATE_VISIBLE))
+			{
+			    event->acc = child;
+			    break;
+			}
+			else
+			    Accessible_unref (child);
+		    }
+		}
+	    }
+	    
 	    AccessibleSelection_unref (selection);
 	}
     }
@@ -1130,7 +1157,11 @@ srl_report_event (SRLEvent *event)
 	ev = srl_last_events[i];
 	srl_last_events[i] = NULL;
 	if (ev)
+	{
+	    if (ev->type == SRL_EVENT_TEXT_CHANGED_INSERT && Accessible_getRole (ev->acc_ev->source) == SPI_ROLE_TERMINAL)
+		srl_notify_clients_obj (ev, SR_EVENT_SRO);
 	    srle_free (ev);
+	}
     }
 	    
     srl_last_events[priority] = srle_dup (event);
@@ -1309,6 +1340,12 @@ srle_change_type (SRLEvent *event)
 	if (srl_acc_has_toolkit (srle_get_acc (event), SRL_TOOLKIT_MOZILLA))
 	    event->type = SRL_EVENT_FOCUS;
     }
+    else if (srle_has_type (event, SRL_EVENT_SELECTION_CHANGED))
+    {
+	if ((Accessible_getRole (event->acc_ev->source) == SPI_ROLE_LAYERED_PANE) &&
+	          (Accessible_getRole (event->acc) == SPI_ROLE_ICON))
+	    event->type = SRL_EVENT_FOCUS;
+    }
     
     if (srle_has_type (event, SRL_EVENT_CHILDREN_ADDED))
 	event->type = SRL_EVENT_TAB_ADDED;
@@ -1566,7 +1603,8 @@ srl_event_is_reported (SRLEvent *event)
 	    srl_last_create = event->acc_ev->source;
 	    Accessible_ref (srl_last_create);
 	}
-	else
+	else if (srle_has_type (event, SRL_EVENT_WINDOW_ACTIVATE) ||
+	    srle_has_type (event, SRL_EVENT_WINDOW_RESTORE))
 	{
 	    if (srl_last_create == event->acc_ev->source)
 		rv = FALSE;
@@ -1776,6 +1814,12 @@ srl_get_context (Accessible *acc)
 
     context = NULL;
     Accessible_ref (acc);
+
+    if (Accessible_getRole (acc) == SPI_ROLE_TABLE && acc != srl_last_table)
+    {
+	context = acc;
+	Accessible_ref (context);
+    }    
     for (i = 0; i < SRL_MAX_LEVELS_UP; i++)
     {
 	Accessible *parent = Accessible_getParent (acc);
@@ -1803,7 +1847,8 @@ srl_get_context (Accessible *acc)
 		    if (label && container)
 		    {
 			if (Accessible_getRole (label) == SPI_ROLE_LABEL &&
-			    Accessible_getRole (container) == SPI_ROLE_FILLER)
+			    (Accessible_getRole (container) == SPI_ROLE_FILLER ||
+			     Accessible_getRole (container) == SPI_ROLE_PANEL))
 			{
 			    gchar *name = Accessible_getName (label);
 			    if (name && name[0])
@@ -1896,10 +1941,9 @@ srl_process_event (SRLEvent *event)
     process = srl_event_is_reported (event);	
     
     if (srle_has_type (event, SRL_EVENT_FOCUS) ||
-	srle_has_type (event, SRL_EVENT_FOCUS2))
+	srle_has_type (event, SRL_EVENT_FOCUS2) ||
+	srle_has_type (event, SRL_EVENT_FOCUS1))
 	    srl_set_last_focus2 ((Accessible*)srle_get_acc (event));
-    else if (srle_has_type (event, SRL_EVENT_FOCUS1))
-	srl_set_last_focus2 (NULL);
 	
     if (process)
 	srl_report_event (event);
@@ -2184,3 +2228,11 @@ srl_is_object_watched (SRObject *obj)
 
     return sro_get_acc (obj) == srl_watched_acc;
 }
+
+gboolean
+srl_is_object_focused (SRObject *obj)
+{
+    srl_assert (obj);
+
+    return sro_get_acc (obj) == srl_last_focus || sro_get_acc (obj) == srl_last_focus2;
+}
